@@ -1,10 +1,10 @@
 /*
- *  $Id: libnet_link_dlpi.c,v 1.1.1.1 2000/05/25 00:28:49 route Exp $
+ *  $Id: libnet_link_dlpi.c,v 1.8 2004/01/28 19:45:00 mike Exp $
  *
  *  libnet
  *  libnet_dlpi.c - dlpi routines
  *
- *  Copyright (c) 1998 - 2001 Mike D. Schiffman <mike@infonexus.com>
+ *  Copyright (c) 1998 - 2004 Mike D. Schiffman <mike@infonexus.com>
  *  All rights reserved.
  *
  * Copyright (c) 1993, 1994, 1995, 1996, 1997
@@ -51,6 +51,10 @@
 #include <sys/systeminfo.h>
 #endif
 
+#ifdef HAVE_SYS_DLPI_EXT_H
+#include <sys/dlpi_ext.h>
+#endif
+
 #ifdef HAVE_HPUX9
 #include <net/if.h>
 #endif
@@ -58,6 +62,7 @@
 #include <ctype.h>
 #ifdef HAVE_HPUX9
 #include <nlist.h>
+#include <dlpi_ext.h>
 #endif
 #include <errno.h>
 #include <fcntl.h>
@@ -83,74 +88,73 @@
 #define	MAXDLBUF 8192
 
 /* Forwards */
-static int dlattachreq(int, bpf_u_int32, char *);
-static int dlbindack(int, char *, char *);
-static int dlbindreq(int, bpf_u_int32, char *);
-static int dlinfoack(int, char *, char *);
-static int dlinforeq(int, char *);
-static int dlokack(int, const char *, char *, char *);
-static int recv_ack(int, int, const char *, char *, char *);
-static int send_request(int, char *, int, char *, char *, int);
+static int dlattachreq(int, bpf_u_int32, int8_t *);
+static int dlbindack(int, int8_t *, int8_t *);
+static int dlbindreq(int, bpf_u_int32, int8_t *);
+static int dlinfoack(int, int8_t *, int8_t *);
+static int dlinforeq(int, int8_t *);
+static int dlokack(int, const int8_t *, int8_t *, int8_t *);
+static int recv_ack(int, int, const int8_t *, int8_t *, int8_t *);
+static int send_request(int, int8_t *, int, int8_t *, int8_t *, int);
 #ifdef HAVE_SYS_BUFMOD_H
-static int strioctl(int, int, int, char *);
+static int strioctl(int, int, int, int8_t *);
 #endif
 #ifdef HAVE_HPUX9
-static int dlpi_kread(int, off_t, void *, u_int, char *);
+static int dlpi_kread(int, off_t, void *, uint, int8_t *);
 #endif
 #ifdef HAVE_DEV_DLPI
-static int get_dlpi_ppa(int, const char *, int, char *);
+static int get_dlpi_ppa(int, const int8_t *, int, int8_t *);
 #endif
 
 /* XXX Needed by HP-UX (at least) */
 static bpf_u_int32 ctlbuf[MAXDLBUF];
 
 
-struct libnet_link_int *
-libnet_open_link_interface(char *device, char *ebuf)
+int
+libnet_open_link(libnet_t *l)
 {
-    register char *cp;
-    char *eos;
-    register struct libnet_link_int *l;
+    register int8_t *cp;
+    int8_t *eos;
     register int ppa;
     register dl_info_ack_t *infop;
     bpf_u_int32 buf[MAXDLBUF];
-    char dname[100];
+    int8_t dname[100];
 #ifndef HAVE_DEV_DLPI
-    char dname2[100];
+    int8_t dname2[100];
 #endif
 
-    l = (struct libnet_link_int *)malloc(sizeof(*l));
     if (l == NULL)
-    {
-        strcpy(ebuf, ll_strerror(errno));
-        return (NULL);
-    }
-    memset(l, 0, sizeof(*l));
+    { 
+        return (-1);
+    } 
 
     /*
      *  Determine device and ppa
      */
-    cp = strpbrk(device, "0123456789");
+    cp = strpbrk(l->device, "0123456789");
     if (cp == NULL)
     {
-        sprintf(ebuf, "%s missing unit number", device);
+        snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
+                "%s(): %s is missing unit number\n", __func__, l->device);
         goto bad;
     }
     ppa = strtol(cp, &eos, 10);
     if (*eos != '\0')
     {
-        sprintf(ebuf, "%s bad unit number", device);
+        snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
+                "%s(): %s bad unit number\n", __func__, l->device);
         goto bad;
     }
 
-    if (*device == '/')
+    if (*(l->device) == '/')
     {
         memset(&dname, 0, sizeof(dname));
-        strncpy(dname, device, sizeof(dname) - 1);
+        strncpy(dname, l->device, sizeof(dname) - 1);
+        dname[sizeof(dname) - 1] = '\0';
     }
     else
     {
-        sprintf(dname, "%s/%s", DLPI_DEV_PREFIX, device);
+        sprintf(dname, "%s/%s", DLPI_DEV_PREFIX, l->device);
     }
 #ifdef HAVE_DEV_DLPI
     /*
@@ -161,14 +165,15 @@ libnet_open_link_interface(char *device, char *ebuf)
     l->fd = open(cp, O_RDWR);
     if (l->fd == -1)
     {
-        sprintf(ebuf, "%s: %s", cp, ll_strerror(errno));
+        snprintf(l->err_buf, LIBNET_ERRBUF_SIZE, "%s(): open(): %s: %s\n",
+                __func__, cp, strerror(errno));
         goto bad;
     }
 
     /*
      *  Map network interface to /dev/dlpi unit
      */
-    ppa = get_dlpi_ppa(l->fd, dname, ppa, ebuf);
+    ppa = get_dlpi_ppa(l->fd, dname, ppa, l->err_buf);
     if (ppa < 0)
     {
         goto bad;
@@ -186,7 +191,8 @@ libnet_open_link_interface(char *device, char *ebuf)
     {
         if (errno != ENOENT)
         {
-            sprintf(ebuf, "%s: %s", dname, ll_strerror(errno));
+            snprintf(l->err_buf, LIBNET_ERRBUF_SIZE, "%s(): open(): %s: %s\n",
+                    __func__, dname, strerror(errno));
             goto bad;
         }
 
@@ -196,13 +202,20 @@ libnet_open_link_interface(char *device, char *ebuf)
         l->fd = open(dname2, O_RDWR);
         if (l->fd == -1)
         {
-            sprintf(ebuf, "%s: %s", dname2, ll_strerror(errno));
+            snprintf(l->err_buf, LIBNET_ERRBUF_SIZE, "%s(): open(): %s: %s\n",
+                    __func__, dname2, strerror(errno));
             goto bad;
         }
 
         cp = dname2;
-        while (*cp && !isdigit((int)*cp)) cp++;
-        if (*cp) ppa = atoi(cp);
+        while (*cp && !isdigit((int)*cp))
+        {
+            cp++;
+        }
+        if (*cp)
+        {
+            ppa = atoi(cp);
+        }
         else
         /*
          *  XXX Assume unit zero
@@ -213,13 +226,15 @@ libnet_open_link_interface(char *device, char *ebuf)
     /*
      *  Attach if "style 2" provider
      */
-    if (dlinforeq(l->fd, ebuf) < 0 || dlinfoack(l->fd, (char *)buf, ebuf) < 0)
+    if (dlinforeq(l->fd, l->err_buf) < 0 ||
+            dlinfoack(l->fd, (int8_t *)buf, l->err_buf) < 0)
     {
         goto bad;
     }
     infop = &((union DL_primitives *)buf)->info_ack;
-    if (infop->dl_provider_style == DL_STYLE2 && (dlattachreq(l->fd, ppa, ebuf)
-        < 0 || dlokack(l->fd, "attach", (char *)buf, ebuf) < 0))
+    if (infop->dl_provider_style == DL_STYLE2 &&
+            (dlattachreq(l->fd, ppa, l->err_buf)
+            < 0 || dlokack(l->fd, "attach", (int8_t *)buf, l->err_buf) < 0))
     {
         goto bad;
     }
@@ -227,8 +242,9 @@ libnet_open_link_interface(char *device, char *ebuf)
     /*
      *  Bind HP-UX 9 and HP-UX 10.20
      */
-#if defined(HAVE_HPUX9) || defined(HAVE_HPUX10_20) || defined(HAVE_SOLARIS)
-    if (dlbindreq(l->fd, 0, ebuf) < 0 || dlbindack(l->fd, (char *)buf, ebuf) < 0)
+#if defined(HAVE_HPUX9) || defined(HAVE_HPUX10_20) || defined(HAVE_HPUX11) || defined(HAVE_SOLARIS)
+    if (dlbindreq(l->fd, 0, l->err_buf) < 0 ||
+            dlbindack(l->fd, (int8_t *)buf, l->err_buf) < 0)
     {
         goto bad;
     }
@@ -237,7 +253,8 @@ libnet_open_link_interface(char *device, char *ebuf)
     /*
      *  Determine link type
      */
-    if (dlinforeq(l->fd, ebuf) < 0 || dlinfoack(l->fd, (char *)buf, ebuf) < 0)
+    if (dlinforeq(l->fd, l->err_buf) < 0 ||
+            dlinfoack(l->fd, (int8_t *)buf, l->err_buf) < 0)
     {
         goto bad;
     }
@@ -247,13 +264,20 @@ libnet_open_link_interface(char *device, char *ebuf)
     {
         case DL_CSMACD:
         case DL_ETHER:
-            l->linktype = DLT_EN10MB;
+            l->link_type    = DLT_EN10MB;
+            l->link_offset  = 0xe;
             break;
         case DL_FDDI:
-            l->linktype = DLT_FDDI;
+            l->link_type    = DLT_FDDI;
+            l->link_offset  = 0x15;
+            break;
+        case DL_TPR:
+            l->link_type    = DLT_PRONET;
+            l->link_offset  = 0x16;
             break;
         default:
-            sprintf(ebuf, "unknown mac type 0x%lu", infop->dl_mac_type);
+            sprintf(l->err_buf, "%s(): unknown mac type 0x%lu\n", __func__,
+                    (uint32_t) infop->dl_mac_type);
             goto bad;
     }
 
@@ -263,20 +287,24 @@ libnet_open_link_interface(char *device, char *ebuf)
      */
     if (strioctl(l->fd, DLIOCRAW, 0, NULL) < 0)
     {
-        sprintf(ebuf, "DLIOCRAW: %s", ll_strerror(errno));
+        sprintf(l->err_buf, "%s(): DLIOCRAW: %s\n", __func__, strerror(errno));
         goto bad;
     }
 #endif
 
-    return (l);
+    return (1);
 bad:
-    free(l);
-    return (NULL);
+    if (l->fd > 0)
+    {
+        close(l->fd);      /* this can fail ok */
+    }
+    return (-1);
 }
 
 
 static int
-send_request(int fd, char *ptr, int len, char *what, char *ebuf, int flags)
+send_request(int fd, int8_t *ptr, int len, int8_t *what, int8_t *ebuf,
+int flags)
 {
     struct strbuf ctl;
 
@@ -286,14 +314,15 @@ send_request(int fd, char *ptr, int len, char *what, char *ebuf, int flags)
 
     if (putmsg(fd, &ctl, (struct strbuf *) NULL, flags) < 0)
     {
-        sprintf(ebuf, "send_request: putmsg \"%s\": %s", what, ll_strerror(errno));
+        sprintf(ebuf, "%s(): putmsg \"%s\": %s\n", __func__, what,
+                strerror(errno));
         return (-1);
     }
     return (0);
 }
 
 static int
-recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
+recv_ack(int fd, int size, const int8_t *what, int8_t *bufp, int8_t *ebuf)
 {
     union DL_primitives *dlp;
     struct strbuf ctl;
@@ -306,7 +335,7 @@ recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
     flags = 0;
     if (getmsg(fd, &ctl, (struct strbuf*)NULL, &flags) < 0)
     {
-        sprintf(ebuf, "recv_ack: %s getmsg: %s", what, ll_strerror(errno));
+        sprintf(ebuf, "%s(): %s getmsg: %s\n", __func__, what, strerror(errno));
         return (-1);
     }
 
@@ -333,7 +362,7 @@ recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
                     break;
                 case DL_SYSERR:
                     sprintf(ebuf, "recv_ack: %s: %s",
-                        what, ll_strerror(dlp->error_ack.dl_unix_errno));
+                        what, strerror(dlp->error_ack.dl_unix_errno));
                     break;
                 case DL_UNSUPPORTED:
                     sprintf(ebuf,
@@ -362,46 +391,23 @@ recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
 }
 
 static int
-dlpromiscoffreq(int fd, bpf_u_int32 level, char *ebuf)
-{
-    dl_promiscon_req_t req;
-
-    req.dl_primitive = DL_PROMISCOFF_REQ;
-    req.dl_level     = level;
-
-    return (send_request(fd, (char *)&req, sizeof(req), "promiscoff", ebuf, 0));
-}
-
-static int
-dlpromisconreq(int fd, bpf_u_int32 level, char *ebuf)
-{
-    dl_promiscon_req_t req;
-
-    req.dl_primitive = DL_PROMISCON_REQ;
-    req.dl_level     = level;
-
-    return (send_request(fd, (char *)&req, sizeof(req), "promiscon", ebuf, 0));
-}
-
-
-static int
-dlattachreq(int fd, bpf_u_int32 ppa, char *ebuf)
+dlattachreq(int fd, bpf_u_int32 ppa, int8_t *ebuf)
 {
     dl_attach_req_t req;
 
     req.dl_primitive = DL_ATTACH_REQ;
     req.dl_ppa       = ppa;
 
-    return (send_request(fd, (char *)&req, sizeof(req), "attach", ebuf, 0));
+    return (send_request(fd, (int8_t *)&req, sizeof(req), "attach", ebuf, 0));
 }
 
 static int
-dlbindreq(int fd, bpf_u_int32 sap, char *ebuf)
+dlbindreq(int fd, bpf_u_int32 sap, int8_t *ebuf)
 {
 
     dl_bind_req_t	req;
 
-    memset((char *)&req, 0, sizeof(req));
+    memset((int8_t *)&req, 0, sizeof(req));
     req.dl_primitive = DL_BIND_REQ;
 #ifdef DL_HP_RAWDLS
     req.dl_max_conind = 1;  /* XXX magic number */
@@ -416,36 +422,34 @@ dlbindreq(int fd, bpf_u_int32 sap, char *ebuf)
     req.dl_service_mode = DL_CLDLS;
 #endif
 #endif
-    return (send_request(fd, (char *)&req, sizeof(req), "bind", ebuf, 0));
+    return (send_request(fd, (int8_t *)&req, sizeof(req), "bind", ebuf, 0));
 }
 
-
 static int
-dlbindack(int fd, char *bufp, char *ebuf)
+dlbindack(int fd, int8_t *bufp, int8_t *ebuf)
 {
     return (recv_ack(fd, DL_BIND_ACK_SIZE, "bind", bufp, ebuf));
 }
 
-
 static int
-dlokack(int fd, const char *what, char *bufp, char *ebuf)
+dlokack(int fd, const int8_t *what, int8_t *bufp, int8_t *ebuf)
 {
     return (recv_ack(fd, DL_OK_ACK_SIZE, what, bufp, ebuf));
 }
 
-
 static int
-dlinforeq(int fd, char *ebuf)
+dlinforeq(int fd, int8_t *ebuf)
 {
     dl_info_req_t req;
 
     req.dl_primitive = DL_INFO_REQ;
 
-    return (send_request(fd, (char *)&req, sizeof(req), "info", ebuf, RS_HIPRI));
+    return (send_request(fd, (int8_t *)&req, sizeof(req), "info", ebuf, 
+            RS_HIPRI));
 }
 
 static int
-dlinfoack(int fd, char *bufp, char *ebuf)
+dlinfoack(int fd, int8_t *bufp, int8_t *ebuf)
 {
     return (recv_ack(fd, DL_INFO_ACK_SIZE, "info", bufp, ebuf));
 }
@@ -453,7 +457,7 @@ dlinfoack(int fd, char *bufp, char *ebuf)
 
 #ifdef HAVE_SYS_BUFMOD_H
 static int
-strioctl(int fd, int cmd, int len, char *dp)
+strioctl(int fd, int cmd, int len, int8_t *dp)
 {
     struct strioctl str;
     int rc;
@@ -476,48 +480,48 @@ strioctl(int fd, int cmd, int len, char *dp)
 #endif
 
 
-#ifdef DL_HP_PPA_ACK_OBS
+#if (defined(DL_HP_PPA_ACK_OBS) && !defined(HAVE_HPUX11))
 /*
  * Under HP-UX 10, we can ask for the ppa
  */
 static int
-get_dlpi_ppa(register int fd, register const char *device, register int unit,
-    register char *ebuf)
+get_dlpi_ppa(register int fd, register const int8_t *device, register int unit,
+register int8_t *ebuf)
 {
     register dl_hp_ppa_ack_t *ap;
     register dl_hp_ppa_info_t *ip;
     register int i;
-    register u_long majdev;
+    register uint32_t majdev;
     dl_hp_ppa_req_t	req;
     struct stat statbuf;
     bpf_u_int32 buf[MAXDLBUF];
 
     if (stat(device, &statbuf) < 0)
     {
-        sprintf(ebuf, "stat: %s: %s", device, ll_strerror(errno));
+        sprintf(ebuf, "stat: %s: %s", device, strerror(errno));
         return (-1);
     }
     majdev = major(statbuf.st_rdev);
 
-    memset((char *)&req, 0, sizeof(req));
+    memset((int8_t *)&req, 0, sizeof(req));
     req.dl_primitive = DL_HP_PPA_REQ;
 
-    memset((char *)buf, 0, sizeof(buf));
-    if (send_request(fd, (char *)&req, sizeof(req), "hpppa", ebuf, 0) < 0 ||
-        recv_ack(fd, DL_HP_PPA_ACK_SIZE, "hpppa", (char *)buf, ebuf) < 0)
+    memset((int8_t *)buf, 0, sizeof(buf));
+    if (send_request(fd, (int8_t *)&req, sizeof(req), "hpppa", ebuf, 0) < 0 ||
+        recv_ack(fd, DL_HP_PPA_ACK_SIZE, "hpppa", (int8_t *)buf, ebuf) < 0)
     {
         return (-1);
     }
 
     ap = (dl_hp_ppa_ack_t *)buf;
-    ip = (dl_hp_ppa_info_t *)((u_char *)ap + ap->dl_offset);
+    ip = (dl_hp_ppa_info_t *)((uint8_t *)ap + ap->dl_offset);
 
     for (i = 0; i < ap->dl_count; i++)
     {
         if (ip->dl_mjr_num == majdev && ip->dl_instance_num == unit)
         break;
 
-        ip = (dl_hp_ppa_info_t *)((u_char *)ip + ip->dl_next_offset);
+        ip = (dl_hp_ppa_info_t *)((uint8_t *)ip + ip->dl_next_offset);
     }
 
     if (i == ap->dl_count)
@@ -547,20 +551,20 @@ static struct nlist nl[] =
     { "" }
 };
 
-static char path_vmunix[] = "/hp-ux";
+static int8_t path_vmunix[] = "/hp-ux";
 
 /*
  *  Determine ppa number that specifies ifname
  */
 static int
-get_dlpi_ppa(register int fd, register const char *ifname, register int unit,
-    register char *ebuf)
+get_dlpi_ppa(register int fd, register const int8_t *ifname, register int unit,
+    register int8_t *ebuf)
 {
-    register const char *cp;
+    register const int8_t *cp;
     register int kd;
     void *addr;
     struct ifnet ifnet;
-    char if_name[sizeof(ifnet.if_name)], tifname[32];
+    int8_t if_name[sizeof(ifnet.if_name)], tifname[32];
 
     cp = strrchr(ifname, '/');
     if (cp != NULL)
@@ -582,7 +586,7 @@ get_dlpi_ppa(register int fd, register const char *ifname, register int unit,
     kd = open("/dev/kmem", O_RDONLY);
     if (kd < 0)
     {
-        sprintf(ebuf, "kmem open: %s", ll_strerror(errno));
+        sprintf(ebuf, "kmem open: %s", strerror(errno));
         return (-1);
     }
 
@@ -613,44 +617,43 @@ get_dlpi_ppa(register int fd, register const char *ifname, register int unit,
 }
 
 static int
-dlpi_kread(register int fd, register off_t addr,
-    register void *buf, register u_int len, register char *ebuf)
+dlpi_kread(register int fd, register off_t addr, register void *buf,
+register uint len, register int8_t *ebuf)
 {
     register int cc;
 
     if (lseek(fd, addr, SEEK_SET) < 0)
     {
-        sprintf(ebuf, "lseek: %s", ll_strerror(errno));
+        sprintf(ebuf, "lseek: %s", strerror(errno));
         return (-1);
     }
     cc = read(fd, buf, len);
     if (cc < 0)
     {
-        sprintf(ebuf, "read: %s", ll_strerror(errno));
+        sprintf(ebuf, "read: %s", strerror(errno));
         return (-1);
     }
     else if (cc != len)
     {
-        sprintf(ebuf, "short read (%d != %d)", cc, len);
+        sprintf(ebuf, "int16_t read (%d != %d)", cc, len);
         return (-1);
     }
     return (cc);
 }
 #endif
 
-/*#include <netinet/if_ether.h>*/
 #define ETHERADDRL 6
 struct  EnetHeaderInfo
 {
-    struct ether_addr   DestEtherAddr;
-    u_short             EtherFrameType;
+    struct libnet_ether_addr   DestEtherAddr;
+    uint16_t EtherFrameType;
 };
 
 
 int
-libnet_close_link_interface(struct libnet_link_int *l)
+libnet_close_link(libnet_t *l)
 {
-    if (close(l->fd) == 0)  
+    if (close(l->fd) == 0)
     {
         return (1);
     }
@@ -660,81 +663,106 @@ libnet_close_link_interface(struct libnet_link_int *l)
     }
 }
 
-
-struct EnetHeaderInfo ArpHeader =
-{
-    {{0xff, 0xff, 0xff, 0xff, 0xff, 0xff}}, ETHERTYPE_ARP
-};
-
-
-int
-libnet_write_link_layer(struct libnet_link_int *l, const char *device,
-            u_char *buf, int len)
-{
+#ifdef HAVE_HPUX11
+int 
+libnet_write_link(libnet_t *l, const uint8_t *packet, uint32_t size)   
+{                                                             
     struct strbuf data, ctl;
-    union DL_primitives *dlp;
+    dl_hp_rawdata_req_t *rdata;
+    int c;                     
+                          
+    if (l == NULL)
+    {             
+        return (-1);           
+    }       
+            
+    rdata = (dl_hp_rawdata_req_t *)ctlbuf;
+    rdata->dl_primitive = DL_HP_RAWDATA_REQ; 
+
+    /* send it */                                  
+    ctl.len = sizeof(dl_hp_rawdata_req_t);                         
+    ctl.maxlen = sizeof(dl_hp_rawdata_req_t);                      
+    ctl.buf = (int8_t *)rdata;
+
+    data.maxlen = size;
+    data.len    = size;                                
+    data.buf    = packet;  
+
+    c = putmsg(l->fd, &ctl, &data, 0);
+    if (c == -1)                      
+    {                    
+        snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
+                "libnet_write_link(): %d bytes written (%s)\n", c,
+                strerror(errno));
+        return (-1);
+    }
+    else
+    {
+        return (size);
+    }                
+}   
+#else
+int
+libnet_write_link(libnet_t *l, const uint8_t *packet, uint32_t size)
+{
     int c;
-    struct EnetHeaderInfo *EnetHeaderInfoP;
+    struct strbuf data;
 
-    dlp = (union DL_primitives*) ctlbuf;
-    dlp->unitdata_req.dl_primitive        = DL_UNITDATA_REQ;
-    dlp->unitdata_req.dl_priority.dl_min  = 0;
-    dlp->unitdata_req.dl_priority.dl_max  = 0;
-    dlp->unitdata_req.dl_dest_addr_length = (sizeof(struct ether_addr) +
-                                            sizeof(u_short));
-    dlp->unitdata_req.dl_dest_addr_offset = DL_UNITDATA_REQ_SIZE;
+    if (l == NULL)
+    { 
+        return (-1);
+    } 
 
-    EnetHeaderInfoP = (struct EnetHeaderInfo *)(ctlbuf + DL_UNITDATA_REQ_SIZE);
-    memcpy(EnetHeaderInfoP, (char *)&(ArpHeader), (sizeof(struct ether_addr) +
-                                                  sizeof(u_short)));
-
-    /* Send it */
-    ctl.len = DL_UNITDATA_REQ_SIZE + sizeof (struct EnetHeaderInfo);
-    ctl.buf = (char *)dlp;
-
-    data.maxlen = len;
-    data.len    = len;
-    data.buf    = buf;
+    data.maxlen = size;
+    data.len    = size;
+    data.buf    = packet;
 
     c = putmsg(l->fd, NULL, &data, 0);
     if (c == -1)
     {
-#if (__DEBUG)
-        fprintf(stderr, "write_link_layer: (%s)\n", strerror(errno));
-#endif
+        snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
+                "libnet_write_link: %d bytes written (%s)\n", c,
+                strerror(errno));
         return (-1);
     }
-    return (len);
+    else
+    {
+        return (size);
+    }
 }
+#endif
 
-
-struct ether_addr *
-libnet_get_hwaddr(struct libnet_link_int *l, const char *device, char *ebuf)
+struct libnet_ether_addr *
+libnet_get_hwaddr(libnet_t *l)
 {
-    char    buf[2048];
+    /* This implementation is not-reentrant. */
+    static int8_t buf[2048];
     union DL_primitives *dlp;
-    struct ether_addr *eap;
+    struct libnet_ether_addr *eap;
 
-    dlp = (union DL_primitives*) buf;
+    if (l == NULL)
+    { 
+        return (NULL);
+    }
+
+    dlp = (union DL_primitives *)buf;
 
     dlp->physaddr_req.dl_primitive = DL_PHYS_ADDR_REQ;
     dlp->physaddr_req.dl_addr_type = DL_CURR_PHYS_ADDR;
 
-    if (send_request(l->fd, (char *)dlp, DL_PHYS_ADDR_REQ_SIZE, "physaddr",
-                    ebuf, 0) < 0)
+    if (send_request(l->fd, (int8_t *)dlp, DL_PHYS_ADDR_REQ_SIZE, "physaddr",
+            l->err_buf, 0) < 0)
     {
-        sprintf(ebuf, "get_hwaddr %s", strerror(errno));
         return (NULL);
     }
-    if (recv_ack(l->fd, DL_PHYS_ADDR_ACK_SIZE, "physaddr", (char *)dlp, ebuf)
-        < 0)
+    if (recv_ack(l->fd, DL_PHYS_ADDR_ACK_SIZE, "physaddr", (int8_t *)dlp,
+            l->err_buf) < 0)
     {
-        sprintf(ebuf, "get_hwaddr %s", strerror(errno));
         return (NULL);
     }
 
-    eap = (struct ether_addr *)
-        ((char *) dlp + dlp->physaddr_ack.dl_addr_offset);
+    eap = (struct libnet_ether_addr *)
+            ((int8_t *) dlp + dlp->physaddr_ack.dl_addr_offset);
     return (eap);
 }   
 
