@@ -39,7 +39,7 @@
 #include "../include/win32/libnet.h"
 #endif
 
-u_int16_t *all_lists;
+uint16_t *all_lists;
 
 int
 libnet_plist_chain_new(libnet_t *l, libnet_plist_t **plist, char *token_list)
@@ -48,8 +48,8 @@ libnet_plist_chain_new(libnet_t *l, libnet_plist_t **plist, char *token_list)
     libnet_plist_t *tmp;
     char *tok;
     int i, j, valid_token, cur_node;
-    u_int16_t *all_lists_tmp;
-    static u_int8_t cur_id;
+    uint16_t *all_lists_tmp;
+    static uint8_t cur_id;
 
     if (l == NULL)
     { 
@@ -101,7 +101,7 @@ libnet_plist_chain_new(libnet_t *l, libnet_plist_t **plist, char *token_list)
     tmp->next = NULL;
     tmp->id = cur_id;
     all_lists_tmp = all_lists;
-    all_lists = realloc(all_lists_tmp, (sizeof(u_int16_t) * (cur_id + 1)));
+    all_lists = realloc(all_lists_tmp, (sizeof(uint16_t) * (cur_id + 1)));
     if (!all_lists)
     {
         all_lists = all_lists_tmp;
@@ -127,7 +127,7 @@ libnet_plist_chain_new(libnet_t *l, libnet_plist_t **plist, char *token_list)
         if (i)
         {
             tmp->next = malloc(sizeof (libnet_plist_t));
-            if (!tmp)
+            if (!tmp->next)
             {
                 snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
                     "libnet_build_plist_chain: malloc %s\n", strerror(errno));
@@ -188,17 +188,17 @@ libnet_plist_chain_new(libnet_t *l, libnet_plist_t **plist, char *token_list)
 }
 
 int
-libnet_plist_chain_next_pair(libnet_plist_t *plist, u_int16_t *bport,
-        u_int16_t *eport)
+libnet_plist_chain_next_pair(libnet_plist_t *plist, uint16_t *bport,
+        uint16_t *eport)
 {
-    u_int16_t *node_cnt;
-    u_int16_t tmp_cnt;
+    uint16_t *node_cnt;
+    uint16_t tmp_cnt;
 
-    node_cnt = &(all_lists[plist->id]);
     if (plist == NULL)
     {
         return (-1);
     }
+    node_cnt = &(all_lists[plist->id]);
 
     /*
      *  We are at the end of the list.
@@ -274,7 +274,7 @@ libnet_plist_chain_dump_string(libnet_plist_t *plist)
 int
 libnet_plist_chain_free(libnet_plist_t *plist)
 {
-    u_int16_t i;
+    uint16_t i;
     libnet_plist_t *tmp;
 
     if (plist == NULL)
